import java.applet.*;
import java.awt.*;
import java.awt.event.*;


public class ChessBoard extends Panel {
///////////////////////////////////////
// chess and board parameters
	private int numGrid = 19;
	private int x0 = 40;
	private int y0 = 45;
	private int gridSize = 32;
	private int chessDiameter = (int)(gridSize * 0.8);
	private int xEnd = x0 + (numGrid-1)*gridSize;
	private int yEnd = y0 + (numGrid-1)*gridSize;
	private int currentXIndex = 0;
	private int currentYIndex = 0;
	private long lastClickTime = 0;
	private long currentTime = 0;

	private TextField text;
  
	private int direction=0;    // 1: horizontally, 2: vertically, 3: diagnally, 4: anti-diagnally
	private int strech1=0, strech2=0;

// several colors used
	private Color play1Color = new Color(50,50,50);
	private Color play1WinColor = new Color(0,0,0);
	private Color play2Color = new Color(200, 200, 200);
	private Color play2WinColor = new Color(255, 255, 255);
	private Color backGroundColor = new Color(255,255,0);

	private String player1Name;
	private String player2Name;

	private int[][] cell = new int[numGrid][numGrid];
  
//several boolean variables
	private boolean isGameOver;
	private boolean isPlay1Turn;
	private boolean isPlay2Turn;
	private boolean isPlay1Win;
	private boolean isPlay2Win;
	private boolean isOutofTime;
  

	public ChessBoard() {
///////////////////////////////////////
// Constructor
  
		player1Name = new String("Player1");
		player2Name = new String("Player2");

		text = new TextField("Now let's play!!           ");
		text.setEditable(false);
		add(text);
		setToInitialState(); 
	}
  
	public void setPlayer1Name(String name) {
		player1Name = name;
	}
  
	public void setPlayer2Name(String name) {
		player2Name = name;
	}

	private void setToInitialState() {
//////////////////////////////////////////
		isPlay1Turn = true;
		isPlay2Turn = false;
		isPlay1Win = false;
		isPlay2Win = false;
		isGameOver = false;
		isOutofTime = false;
		setBackground(backGroundColor);

		for(int i=0; i<numGrid; i++)
			for(int j=0; j<numGrid; j++)
				cell[i][j] = 0;
	}

	public void reset() {
/////////////////////////////////////////////////
// start new game
		text.setText("Now let's play!!           ");
		setToInitialState();
		repaint();
	}

	public void undo() {
//////////////////////////////////////
// undo one step
		if(!isGameOver && cell[currentXIndex][currentYIndex]!=0) {
			if(!isPlay1Turn) text.setText("Hei, it's your turn now -- " + player1Name);
			else           text.setText("Hei, it's your turn now -- " + player2Name);
        
			cell[currentXIndex][currentYIndex] = 0;
			swapTurn();
			repaint();
		}
	}


public boolean isSomeoneWin() {
///////////////////////////////////////
// most important algorithm here

	int t1=0;
	int t2=0;
       
	int i=currentXIndex;
	int j=currentYIndex;
  
	if(!isPlay1Turn) {
      
      // first path: horizontally
		for(int k=1;k<5; k++) {
			if(isInBound(i+k,j) &&  cell[i+k][j] == 1) t1++;
			else break;
		}
		for(int k=1;k<5; k++) {
			if(isInBound(i-k,j) && cell[i-k][j] ==1) t2++;
			else break;
		}
		if(t1+t2+1>=5) { isPlay1Win = true; direction=1; strech1=t1; strech2=t2; return true;}
		else { t1=0; t2=0;}

      //second path: vertically
		for(int k=1;k<5; k++) {
			if(isInBound(i,j+k) &&  cell[i][j+k] == 1) t1++;
			else break;
		}
		for(int k=1;k<5; k++) {
			if(isInBound(i,j-k) && cell[i][j-k] ==1) t2++;
			else break;
		}
		if(t1+t2+1>=5) { isPlay1Win=true; direction=2; strech1=t1; strech2=t2; return true; }
		else { t1=0; t2=0;}

      //third path: diagnally
		for(int k=1;k<5; k++) {
			if(isInBound(i+k,j+k) &&  cell[i+k][j+k] == 1) t1++;
			else break;
		}
		for(int k=1;k<5; k++) {
			if(isInBound(i-k,j-k) && cell[i-k][j-k] ==1) t2++;
			else break;
		}
		if(t1+t2+1>=5) { isPlay1Win=true; direction=3; strech1=t1; strech2=t2; return true; }
		else { t1=0; t2=0;}

      //forth path: anti-diagnally
		for(int k=1;k<5; k++) {
			if(isInBound(i+k,j-k) &&  cell[i+k][j-k] == 1) t1++;
			else break;
		}
		for(int k=1;k<5; k++) {
			if(isInBound(i-k,j+k) && cell[i-k][j+k] ==1) t2++;
			else break;
		}
		if(t1+t2+1>=5) { isPlay1Win=true; direction=4; strech1=t1; strech2=t2; return true; }
		else { t1=0; t2=0;}
	}
    
	else {
      // first path: horizontally
		for(int k=1;k<5; k++) {
			if(isInBound(i+k,j) &&  cell[i+k][j] == 2) t1++;
			else break;
		}
		for(int k=1;k<5; k++) {
			if(isInBound(i-k,j) && cell[i-k][j] ==2) t2++;
			else break;
		}
		if(t1+t2+1>=5) {isPlay2Win=true;  direction=1; strech1=t1; strech2=t2;  return true; }
		else { t1=0; t2=0;}

      //second path: vertically
		for(int k=1;k<5; k++) {
			if(isInBound(i,j+k) &&  cell[i][j+k] == 2) t1++;
			else break;
		}
		for(int k=1;k<5; k++) {
			if(isInBound(i,j-k) && cell[i][j-k] ==2) t2++;
			else break;
		}
		if(t1+t2+1>=5) {isPlay2Win=true; direction=2; strech1=t1; strech2=t2; return true; }
		else { t1=0; t2=0;}

      //third path: diagnally
		for(int k=1;k<5; k++) {
			if(isInBound(i+k,j+k) &&  cell[i+k][j+k] == 2) t1++;
			else break;
		}
		for(int k=1;k<5; k++) {
			if(isInBound(i-k,j-k) && cell[i-k][j-k] ==2) t2++;
			else break;
		}
		if(t1+t2+1>=5) {isPlay2Win=true;  direction=3; strech1=t1; strech2=t2; return true; }
		else { t1=0; t2=0;}

      //forth path: anti-diagnally
		for(int k=1;k<5; k++) {
			if(isInBound(i+k,j-k) &&  cell[i+k][j-k] == 2) t1++;
			else break;
		}
		for(int k=1;k<5; k++) {
			if(isInBound(i-k,j+k) && cell[i-k][j+k] ==2) t2++;
			else break;
		}
		if(t1+t2+1>=5) {isPlay2Win=true; direction=4; strech1=t1; strech2=t2; return true; }
		else { t1=0; t2=0;}
	} 
	return false;
}  


	private boolean isInBound(int i, int j) {
/////////////////////////////////////////////////////  
		if(i>=0 && i<=numGrid-1 && j>=0 && j<=numGrid-1) 
			return true;
		else
		return false;
	}



	private void swapTurn() {
////////////////////////////////////////////
		isPlay1Turn = !isPlay1Turn; 
		isPlay2Turn = !isPlay2Turn;
	}


	public boolean mouseDown(Event e, int x, int y) {
///////////////////////////////////////////
		int i = Math.round((x-x0)/(float)(gridSize));
		int j = Math.round((y-y0)/(float)(gridSize));
   
   
		if(!isGameOver && isInBound(i,j) && cell[i][j]==0) {
			if(!isPlay1Turn) text.setText("Hei, it's your turn now -- " + player1Name);
			else           text.setText("Hei, it's your turn now -- " + player2Name);
       
			currentXIndex = i;
			currentYIndex = j;
             
			if(isPlay1Turn) {
				cell[i][j] = 1;
				swapTurn();
			}     
			else {
				cell[i][j] = 2;
				swapTurn();
			}
          
			repaint();
		}
		return true;
	}
    

	public void drawChess(Graphics g) {
/////////////////////////////////////////////////
		for(int i=0; i<numGrid; i++) {
			for(int j=0; j<numGrid; j++) {
				if(isInBound(i,j) && cell[i][j]!=0) { 
					switch(cell[i][j]) {
						case 1: {
							g.setColor(play1Color);
					    	break;
						 }
						case 2: {
							g.setColor(play2Color);
							break;
						}
						case 3: {
							g.setColor(play1WinColor);
							break;
						} 
						case 4: {
							g.setColor(play2WinColor);
                     		break;
						}
					}
					g.fillOval(x0+i*gridSize-chessDiameter/2, y0+j*gridSize-chessDiameter/2, chessDiameter, chessDiameter);
				}
			}
		}
	}
  


    
	private void pause(int time) {
		try{
			Thread.sleep(time);
		} catch(InterruptedException e) {}
	}
 

	private void showSomeWon(Graphics g) {
/////////////////////////////////////////////////////////////////
    
		if(isPlay1Win) {
			text.setText("You WIN!!! --- " + player1Name);
			for(int i=0; i<6; i++) {
				recoverIt(1);
				drawChess(g);
				pause(250);
				recoverIt(3);
				drawChess(g);
				pause(250);
			}
		}
		else {
			text.setText("You WIN!!! --- " + player2Name);
			for(int i=0; i<6; i++) {
				recoverIt(2);
				drawChess(g);
				pause(250);
				recoverIt(4);
				drawChess(g);
				pause(250);
        
			}
		}
		isPlay1Win = false;
		isPlay2Win = false;
	}
  

	private void makeItZero() {
//////////////////////////////////////////////////////////
		recoverIt(0); 
	}


	public void recoverIt(int t) {
//////////////////////////////////////////////////////////
		int i=currentXIndex;
		int j=currentYIndex;
 
		cell[i][j]=t;

		switch(direction) {
			case 1: {  
                if(strech1!=0) 
                  for(int k=1; k<=strech1; k++) cell[i+k][j]=t;
                if(strech2!=0)
                  for(int k=1; k<=strech2; k++) cell[i-k][j]=t;
                break;
              }
			case 2: {  
                if(strech1!=0) 
                  for(int k=1; k<=strech1; k++) cell[i][j+k]=t;
                if(strech2!=0)
                  for(int k=1; k<=strech2; k++) cell[i][j-k]=t;
                break;
              }
			case 3: {  
                if(strech1!=0) 
                  for(int k=1; k<=strech1; k++) cell[i+k][j+k]=t;
                if(strech2!=0)
                  for(int k=1; k<=strech2; k++) cell[i-k][j-k]=t;
                break;
              } 
			case 4: {  
                if(strech1!=0) 
                  for(int k=1; k<=strech1; k++) cell[i+k][j-k]=t;
                if(strech2!=0)
                  for(int k=1; k<=strech2; k++) cell[i-k][j+k]=t;
                break;
              }  
		}
	}


	public void paint(Graphics g) {
/////////////////////////////////////////////////////////////
   		drawBoard(g);
		if(isSomeoneWin()) {showSomeWon(g); isGameOver = true;}
		drawChess(g);
	}
  
	public void draw() {
		repaint();
	} 



	private void drawBoard(Graphics g) {
//////////////////////////////////////////////////////
		for(int i=0; i<numGrid; i++) {
			int currentX = x0 + i*gridSize;
			int currentY = y0 + i*gridSize;
			g.drawLine(x0, currentY, xEnd, currentY);
			g.drawLine(currentX, y0, currentX, yEnd);
		}
		g.drawRect(x0-5, y0-5, xEnd-x0+10, yEnd-y0+10);
		g.setColor(Color.black);
		g.fillOval(x0+3*gridSize-4, y0+3*gridSize-4,8,8);
		g.fillOval(x0+3*gridSize-4, (y0+yEnd)/2-4,8,8);
		g.fillOval(x0+3*gridSize-4, yEnd-3*gridSize-4,8,8);
		g.fillOval((x0+xEnd)/2-4, y0+3*gridSize-4,8,8);
		g.fillOval(xEnd-3*gridSize-4, y0+3*gridSize-4,8,8);
		g.fillOval(xEnd-3*gridSize-4, (y0+yEnd)/2-4,8,8);
		g.fillOval(xEnd-3*gridSize-4, yEnd-3*gridSize-4,8,8);
		g.fillOval((x0+xEnd)/2-4, yEnd-3*gridSize-4,8,8);
		g.fillOval((x0+xEnd)/2-4, (y0+yEnd)/2-4,8,8);
	}
} 
